require "import"
import "android.media.MediaRecorder"
import "java.io.File"
import "java.text.SimpleDateFormat"
import "java.util.Date"
import "android.content.Context"
import "android.os.Vibrator"
import "android.content.Intent"
import "android.net.Uri"

-- Global variables to maintain state
if not GLOBAL_STATE then
    GLOBAL_STATE = {
        isRecording = false,
        mediaRecorder = nil,
        outputFile = nil
    }
end

local function vibrate(duration)
    local context = activity or service
    local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
    if vibrator and vibrator.hasVibrator() then
        vibrator.vibrate(duration)
    end
end

local function shareAudioFile(filePath)
    if filePath and File(filePath).exists() then
        local intent = Intent(Intent.ACTION_SEND)
        intent.setType("audio/mp3")
        
        -- Use content provider to get a shareable URI
        local uri = service.getUriForFile(File(filePath))
        
        intent.putExtra(Intent.EXTRA_STREAM, uri)
        intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
        
        local chooserIntent = Intent.createChooser(intent, "Share audio recording")
        chooserIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        
        service.startActivity(chooserIntent)
    else
        service.asyncSpeak("No audio file to share")
    end
end

local function startRecording()
    if GLOBAL_STATE.isRecording then
        service.asyncSpeak("Recording is already in progress")
        return
    end

    local folderName = "Global Recorder"
    local folderPath = "/storage/emulated/0/" .. folderName
    local folder = File(folderPath)
    if not folder.exists() then
        local success = folder.mkdirs()
        if not success then
            service.asyncSpeak("Could not create the folder at: " .. folderPath)
            return
        end
    end

    local dateFormat = SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault())
    local currentTime = dateFormat.format(Date())
    GLOBAL_STATE.outputFile = folderPath .. "/recorded_audio_" .. currentTime .. ".mp3"

    GLOBAL_STATE.mediaRecorder = MediaRecorder()
    GLOBAL_STATE.mediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC)
    GLOBAL_STATE.mediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4)
    GLOBAL_STATE.mediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC)
    GLOBAL_STATE.mediaRecorder.setAudioEncodingBitRate(96000)
    GLOBAL_STATE.mediaRecorder.setAudioSamplingRate(44100)
    GLOBAL_STATE.mediaRecorder.setOutputFile(GLOBAL_STATE.outputFile)

    GLOBAL_STATE.mediaRecorder.prepare()
    GLOBAL_STATE.mediaRecorder.start()
    GLOBAL_STATE.isRecording = true

    service.asyncSpeak("Recording started")
    vibrate(100)
end

local function stopRecording()
    if not GLOBAL_STATE.isRecording then
        service.asyncSpeak("No active recording to stop")
        return
    end

    GLOBAL_STATE.mediaRecorder.stop()
    GLOBAL_STATE.mediaRecorder.release()
    GLOBAL_STATE.mediaRecorder = nil
    GLOBAL_STATE.isRecording = false

    service.asyncSpeak("Recording finished")
    vibrate(200)
    
    -- Add a small delay to ensure the file is properly saved
    os.execute("sleep " .. tonumber(1))
    
    -- Share the recorded audio file
    shareAudioFile(GLOBAL_STATE.outputFile)
end

-- Main execution
if GLOBAL_STATE.isRecording then
    stopRecording()
else
    startRecording()
end

return true